﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Media;
using System.Windows.Threading;
using BezierSpline.Models;

namespace BezierSpline.ViewModels
{
    public class ViewModel
    {
        private double _screenWidth, _screenHeight;
        private SelectedColorModel[] _availableColors = new SelectedColorModel[] { Colors.Red, Colors.Blue, Colors.Green, Colors.Orange, Colors.White };
        private Dispatcher _dispatcher;
        private DispatcherTimer _timer;
        private ObservableCollection<ClusterModel> _clusters;

        public int[] AvailableClusters { get { return new int[] { 1, 2, 3 }; } }
        public int[] AvailableRings { get { return new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 }; } }
        public int[] AvailableInflexions { get { return new int[] { 3, 4, 5, 6, 7 }; } }
        public int[] AvailableSpeeds { get { return new int[] { 1, 2, 3, 4 }; } }
        public SelectedColorModel[] AvailableColors { get { return _availableColors; } }

        public int ClusterCount { get; set; }
        public int RingCount { get; set; }
        public int InflexionCount { get; set; }
        public int Speed { get; set; }

        public ObservableCollection<ClusterModel> Clusters { get { return _clusters; } }

        public ViewModel(Dispatcher dispatcher)
        {
            _dispatcher = dispatcher;

            ClusterCount = 1;
            RingCount = 5;
            InflexionCount = 4;
            Speed = 2;

            _clusters = new ObservableCollection<ClusterModel>();
            _timer = new DispatcherTimer();
            _timer.Interval = TimeSpan.FromSeconds(1.0 / 40);
            _timer.Tick += (ss, ee) =>
            {
                for (int i = 0; i < ClusterCount - _clusters.Count; i++)
                    _clusters.Add(new ClusterModel());

                for (int i = 0; i < _clusters.Count; )
                {
                    _clusters[i].SetScreenSize(_screenWidth, _screenHeight);
                    _clusters[i].SetAvailableColors(_availableColors.Where(f => f.IsChecked).Select(s => s.ColorName).ToArray());
                    _clusters[i].Step(Speed * 3);
                    _clusters[i].Fix(RingCount, InflexionCount);

                    if (i < ClusterCount)
                        i++;
                    else
                        _clusters.RemoveAt(i);
                }
            };
            _timer.Start();
        }

        public void SetScreenSize(double screenWidth, double screenHeight)
        {
            _screenWidth = screenWidth;
            _screenHeight = screenHeight;
        }
    }
}
